<?php

namespace Drupal\fillpdf_comprehensive_mapper;

use Drupal\fillpdf\Entity\FillPdfForm;

/**
 * Methods for running the bulk update of all other FillPDF maps in a batch.
 */
class BulkUpdateBatch {

  /**
   * Processes the config import batch and persists the importer.
   *
   * @param int $fillpdf_id;
   *   The form to use as the source for all others.
   * @param array $context
   *   The batch context.
   */
  public static function process(int $batch_id, array $fids, int $source_fid, array &$context): void {
    if (!isset($context['sandbox']['progress'])) {
      $context['sandbox']['progress'] = 0;
      $context['sandbox']['current_fid'] = 0;
      $context['sandbox']['max'] = 0;
    }
    if (!isset($context['results']['updated'])) {
      $context['results']['updated'] = 0;
      $context['results']['skipped'] = 0;
      $context['results']['failed'] = 0;
      $context['results']['progress'] = 0;
    }
    // Total records to process for all batches.
    if (empty($context['sandbox']['max'])) {
      $total = \Drupal::entityQuery('fillpdf_form')->accessCheck(TRUE)->execute();
      $context['sandbox']['max'] = count($total) - 1;
    }
    // Load up the source form fields that will overwrite all the others once.
    if (!isset($context['results']['source_form_fields'])) {
      $sourceFillPdfForm = FillPdfForm::load($source_fid);
      $context['results']['source_form_fields'] = $sourceFillPdfForm->getFormFields();
    }

    // Keep track of progress.
    $context['results']['progress'] += count($fids);
    $context['results']['process'] = 'Import request files';
    // Message above progress bar.
    $context['message'] = t('Processing batch #@batch_id batch size @batch_size for total @count items.',[
      '@batch_id' => number_format($batch_id),
      '@batch_size' => number_format(count($fids)),
      '@count' => number_format($context['sandbox']['max']),
    ]);

    $serializer = \Drupal::service('fillpdf.serializer');
    foreach ($fids as $fid) {
      $destFillPdfForm = FillPdfForm::load($fid);
      $existing_fields = $destFillPdfForm->getFormFields();
      $unmatched_pdf_keys = $serializer->importFormFields($context['results']['source_form_fields'], $existing_fields);
    }

  }

  /**
   * Finish batch.
   *
   * This function is a static function to avoid serializing the ConfigSync
   * object unnecessarily.
   *
   * @param bool $success
   *   Indicate that the batch API tasks were all completed successfully.
   * @param array $results
   *   An array of all the results that were updated in update_do_one().
   * @param array $operations
   *   A list of the operations that had not been completed by the batch API.
   */
  public static function finish($success, $results, $operations) {
    // Unset data we were only stashing in results for convenience.
    unset($results['source_form_fields']);
    $messenger = \Drupal::messenger();
    if ($success) {
      if (!empty($results['errors'])) {
        $logger = \Drupal::logger('fillpdf_comprehensive_mapper');
        foreach ($results['errors'] as $error) {
          $messenger->addError($error);
          $logger->error($error);
        }
        $messenger->addWarning(t('Form fields configured on the source comprehensive mapper FillPDF form have overwritten all other FillPDF forms, but with errors.'));
      }
      else {
        $messenger->addStatus(t('Form fields configured on the source comprehensive mapper FillPDF form have overwritten all other FillPDF forms.'));
      }
    }
    else {
      // An error occurred.
      // $operations contains the operations that remained unprocessed.
      $error_operation = reset($operations);
      $message = t('An error occurred while processing %error_operation with arguments: @arguments', ['%error_operation' => $error_operation[0], '@arguments' => print_r($error_operation[1], TRUE)]);
      $messenger->addError($message);
    }
  }

}